// igDateEditor extension
/*global WinJS, $*/
(function () {
    WinJS.Namespace.define('Infragistics.UI', {
        DateEditor: WinJS.Class.define(function (element, options) {
			// This needs to be defined for all widgets
            if (!element) {
                throw new Error("The Infragistics control requires a DOM element to be instantiated on.");
            }
			this._element = element;
			// Initializes the control
			MSApp.execUnsafeLocalFunction(function () {
				$(element).igDateEditor(options);
			});
			this.element.winControl = this;
        }, {
			// Private properties
			_element: null,
            // Public options
			// The element option is generic for all widget and is only a getter
			element: {
                get: function () {
                    return this._element;
                }
            },
						/// <field type="Object">
			/// Sets gets custom regional settings for editor. If it is string, then $.ig.regional[stringValue] is assumed.
			/// </field>
			regional: {
				get: function () {
					return $(this._element).igDateEditor('option', 'regional');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'regional', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to modify only 1 date field on spin events.
///				Value false enables changes of other date fields when incremented or decremented date-field reaches its limits.
///				Value true modifies only value of one field.
			/// </field>
			spin1Field: {
				get: function () {
					return $(this._element).igDateEditor('option', 'spin1Field');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'spin1Field', value);
				}
			},
			/// <field type="Date">
			/// Sets gets the minimum value which can be entered in editor by user.
			/// </field>
			minValue: {
				get: function () {
					return $(this._element).igDateEditor('option', 'minValue');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'minValue', value);
				}
			},
			/// <field type="Date">
			/// Sets gets the maximum value which can be entered in editor by user.
			/// </field>
			maxValue: {
				get: function () {
					return $(this._element).igDateEditor('option', 'maxValue');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'maxValue', value);
				}
			},
			/// <field type="String">
			/// Sets gets format of date while editor has no focus.
///				Value of that option can be set to a specific date pattern or to a flag defined by regional settings.
///				If value is not set, then the dateInputFormat is used automatically.
///				If value is set to explicit date pattern and pattern besides date-flags has explicit characters which match with date-flags or mask-flags, then the "escape" character should be used in front of them. 
///				List of predefined regional flags:
///				"date": the datePattern member of regional option is used
///				"dateLong": the dateLongPattern member of regional option is used
///				"time": the timePattern member of regional option is used
///				"timeLong": the timeLongPattern member of regional option is used
///				"dateTime": the dateTimePattern member of regional option is used
///				List of explicit characters, which should have escape \\ character in front of them:
///					C, &, a, A, ?, L, 9, 0, #, >, <, y, M, d, h, H, m, s, t, f.
///				List of date-flags when explicit date pattern is used:
///				"y": year field without century and without leading zero
///				"yy": year field without century and with leading zero
///				"yyyy": year field with leading zeros
///				"M": month field as digit without leading zero
///				"MM": month field as digit with leading zero
///				"MMM": month field as short month name
///				"MMMM": month field as long month name
///				"d": day of month field without leading zero 
///				"dd": day of month field with leading zero
///				"ddd": day of the week as short name
///				"dddd": day of the week as long name
///				"t": first character of string which represents AM/PM field 
///				"tt": 2 characters of string which represents AM/PM field
///				"h": hours field in 12-hours format without leading zero
///				"hh": hours field in 12-hours format with leading zero
///				"H": hours field in 24-hours format without leading zero
///				"HH": hours field in 24-hours format with leading zero
///				"m": minutes field without leading zero
///				"mm": minutes field with leading zero
///				"s": seconds field without leading zero
///				"ss": seconds field with leading zero
///				"f": milliseconds field in thousands
///				"ff": milliseconds field in hundreds
///				"fff": milliseconds field
			/// </field>
			dateDisplayFormat: {
				get: function () {
					return $(this._element).igDateEditor('option', 'dateDisplayFormat');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'dateDisplayFormat', value);
				}
			},
			/// <field type="String">
			/// Sets gets format of date while editor has focus.
///				Value of that option can be set to explicit date pattern or to a flag defined by regional settings.
///				If value is set to explicit date pattern and pattern besides date-flags has explicit characters which match with date-flags or mask-flags, then the "escape" character should be used in front of them.
///				If option is not set, then the "date" is used automatically.
///				List of predefined regional flags:
///				"date": the datePattern member of regional option is used
///				"dateLong": the dateLongPattern member of regional option is used
///				"time": the timePattern member of regional option is used
///				"timeLong": the timeLongPattern member of regional option is used
///				"dateTime": the dateTimePattern member of regional option is used
///				List of explicit characters, which should have escape \\ character in front of them: C, &, a, A, ?, L, 9, 0, #, >, <, y, M, d, h, H, m, s, t, f.
///				List of date-flags when explicit date pattern is used:
///				"y": year field without century and without leading zero
///				"yy": year field without century and with leading zero
///				"yyyy": year field with leading zeros
///				"M": month field as digit without leading zero
///				"MM": month field as digit with leading zero
///				"MMM": month field as short month name. Note: in focused state the MM is used.
///				"MMMM": month field as long month name. Note: in focused state the MM is used.
///				"d": day of month field without leading zero 
///				"dd": day of month field with leading zero
///				"ddd": day of the week as short name. Note: in focused state that field is skipped.
///				"dddd": day of the week as long name. Note: in focused state that field is skipped.
///				"t": first character of string which represents AM/PM field 
///				"tt": 2 characters of string which represents AM/PM field
///				"h": hours field in 12-hours format without leading zero
///				"hh": hours field in 12-hours format with leading zero
///				"H": hours field in 24-hours format without leading zero
///				"HH": hours field in 24-hours format with leading zero
///				"m": minutes field without leading zero
///				"mm": minutes field with leading zero
///				"s": seconds field without leading zero
///				"ss": seconds field with leading zero
///				"f": milliseconds field in thousands
///				"ff": milliseconds field in hundreds
///				"fff": milliseconds field
			/// </field>
			dateInputFormat: {
				get: function () {
					return $(this._element).igDateEditor('option', 'dateInputFormat');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'dateInputFormat', value);
				}
			},
			/// <field type="String">
			/// Sets gets character which is used as prompt in edit mode for available entry position. Default is "_"
			/// </field>
			promptChar: {
				get: function () {
					return $(this._element).igDateEditor('option', 'promptChar');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'promptChar', value);
				}
			},
			/// <field type="Enumeration">
			///  Sets gets type of value returned by the get of value() method. That also affects functionality of the set value(val) method and the copy/paste operations of browser.
			/// </field>
			dataMode: {
				get: function () {
					return $(this._element).igDateEditor('option', 'dataMode');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'dataMode', value);
				}
			},
			/// <field type="Number">
			/// Sets gets the minimum number of date fields, which makes entered date-string valid.
///				All missing fields are automatically filled up from the previous valid date or with zeros or with the values from the today date.
///				For example, if "dateInputPattern" is set to "yyyy/MM/dd hh:mm" and user entered only the first 3 fields (year, month, day), then values for hours and minutes are filled with zeros.
			/// </field>
			minNumberOfDateFields: {
				get: function () {
					return $(this._element).igDateEditor('option', 'minNumberOfDateFields');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'minNumberOfDateFields', value);
				}
			},
			/// <field type="Bool">
			/// Set gets ability to use last valid date when user entered invalid value.
///				Value of true allows automatically use last valid date, when editor loses focus and user failed to enter a valid date.
///				In case of value false, the entered date is considered invalid and null is used.
			/// </field>
			useLastGoodDate: {
				get: function () {
					return $(this._element).igDateEditor('option', 'useLastGoodDate');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'useLastGoodDate', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to decrease day of month.
///				Value of true allows decreasing day of month, when editor loses focus and user entered day of month which is larger the maximum for that particular month.
///				Value of false will automatically increment month and decrease day of month.
///				Examples.
///				If that option is true and user entered "09/31/2010", then on lost focus that string will be converted to "09/30/2010".
///				If that option is false, then in similar situation the string will be converted to "10/01/2010".
			/// </field>
			reduceDayOfInvalidDate: {
				get: function () {
					return $(this._element).igDateEditor('option', 'reduceDayOfInvalidDate');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'reduceDayOfInvalidDate', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to hide mask in focus state.
///				Value of true allows showing or hiding all input mask with prompt characters regardless if available mask positions are filled or not.
///				Value of true will show only part of mask located on the left side from caret when available mask positions on the right side from caret are empty.
			/// </field>
			hideMaskOnFocus: {
				get: function () {
					return $(this._element).igDateEditor('option', 'hideMaskOnFocus');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'hideMaskOnFocus', value);
				}
			},
			/// <field type="Number">
			/// Sets gets year for auto detection of 20th and 21st centuries.
///				That option is used to automatically fill century when the user entered only 1 or 2 digits into the year field or when the date pattern contains only 1 or 2 year positions, e.g. "yy" or "y".
///				If user entered value larger than value of this option, then 20th century is used, otherwise the 21st.
			/// </field>
			centuryThreshold: {
				get: function () {
					return $(this._element).igDateEditor('option', 'centuryThreshold');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'centuryThreshold', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets formatting of the dates as UTC.
///				That option is supported only when dataMode option is 'date' and Date objects are used to get/set value of editor.
///				Notes:
///				That option affects only functionality of get/set value method and the Date-value, which was set on initialization.
///				When application uses the set-value, then internal Date-value and displayed-text is incremented by TimezoneOffset.
///				When application uses the get-value, then editor returns internal Date-value decremented by TimezoneOffset.
///				When that option is modified after initialization, then displayed text and internal Date-value are not affected.
///				It is not recommended to change that option without resetting Date-value.
			/// </field>
			enableUTCDates: {
				get: function () {
					return $(this._element).igDateEditor('option', 'enableUTCDates');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'enableUTCDates', value);
				}
			},
			/// <field type="Number">
			/// Sets gets delta-value which is used to increment or decrement the value of field where caret is located is modified.
			/// </field>
			spinDelta: {
				get: function () {
					return $(this._element).igDateEditor('option', 'spinDelta');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'spinDelta', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to prevent null value.
///				If that option is disabled, and editor has no value, then value is set to today date (or minValue/maxValue).
			/// </field>
			nullable: {
				get: function () {
					return $(this._element).igDateEditor('option', 'nullable');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'nullable', value);
				}
			},
			/// <field type="Number">
			/// Sets gets difference between year in Gregorian calendar and displayed year.
			/// </field>
			yearShift: {
				get: function () {
					return $(this._element).igDateEditor('option', 'yearShift');
				},
				set: function (value) {
					$(this._element).igDateEditor('option', 'yearShift', value);
				}
			},

			// Public methods
			/*
			destroy: function igDateEditor_destroy() {
				$(this._element).igDateEditor('destroy');
			}
			*/
			
        }, {
			// Static members
		})
    });
}());