// igEditor extension
/*global WinJS, $*/
(function () {
    WinJS.Namespace.define('Infragistics.UI', {
        Editor: WinJS.Class.define(function (element, options) {
			// This needs to be defined for all widgets
            if (!element) {
                throw new Error("The Infragistics control requires a DOM element to be instantiated on.");
            }
			this._element = element;
			// Initializes the control
			MSApp.execUnsafeLocalFunction(function () {
				$(element).igEditor(options);
			});
			this.element.winControl = this;
        }, {
			// Private properties
			_element: null,
            // Public options
			// The element option is generic for all widget and is only a getter
			element: {
                get: function () {
                    return this._element;
                }
            },
						/// <field type="Object">
			/// Gets sets value in editor. The effect of setting/getting that option depends on type of editor and on dataMode options. That can be string, number or Date depending on type of editor. If it is used on initialization and the type option is missing, then if 'value' is Number, then 'numeric' editor is created automatically and if 'value' is Date, then the 'date' editor is created.
			/// </field>
			value: {
				get: function () {
					return $(this._element).igEditor('option', 'value');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'value', value);
				}
			},
			/// <field type="Number">
			/// Gets sets value in tabIndex for editor.
			/// </field>
			tabIndex: {
				get: function () {
					return $(this._element).igEditor('option', 'tabIndex');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'tabIndex', value);
				}
			},
			/// <field type="String">
			/// Sets gets text which appears in editor when editor has no focus and "value" in editor is null or empty string.
			/// </field>
			nullText: {
				get: function () {
					return $(this._element).igEditor('option', 'nullText');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'nullText', value);
				}
			},
			/// <field type="Enumeration">
			///  Sets gets visibility of spin and drop-down button. That option can be set only on initialization. Combinations like 'dropdown,spin' or 'spinclear' are supported too.
			/// </field>
			button: {
				get: function () {
					return $(this._element).igEditor('option', 'button');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'button', value);
				}
			},
			/// <field type="Enumeration">
			///  Sets gets horizontal alignment of text in editor. If that option is not set, then 'right' is used for 'numeric', 'currency' and 'percent' editors and the 'left' is used for all other types of editor.
			/// </field>
			textAlign: {
				get: function () {
					return $(this._element).igEditor('option', 'textAlign');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'textAlign', value);
				}
			},
			/// <field type="Array">
			/// Sets gets list of items which are used for drop-down list, spin, validation and auto-complete functionality.
///				Items in list can be strings, numbers, dates or objects in any combination.
///				If type of editor is date or datepicker and item is string, then igEditor will try to convert it to Date object and show item in display format.
///				If type of editor is numeric, currency or percent and item is string, then igEditor will try to convert it to number and show item in display format.
///				If item is object and it has member "text", then that member is used.
///				If item is object and besides "text" has function getHtml(), then that function is used to render item in list.
///				The item or item.text is used to set "value" of particular editor when list-item is selected.
			/// </field>
			listItems: {
				get: function () {
					return $(this._element).igEditor('option', 'listItems');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'listItems', value);
				}
			},
			/// <field type="String">
			/// Sets gets selector for css classes used by editor.
///				That option allows replacing all default css styles of editor by custom values.
///				Application should provide css classes for all members defined in the css options with "theme" selector.
///				For example, if that property is set to "mytheme", then application should provide following css classes:
///				.mytheme .ui-igedit-field{...};
///				.mytheme .ui-igedit-focus{...};
///				etc.
///				Note: changing theme is not supported when base element is INPUT or TEXTAREA and renderInContainer or button are not enabled.
			/// </field>
			theme: {
				get: function () {
					return $(this._element).igEditor('option', 'theme');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'theme', value);
				}
			},
			/// <field type="Enumeration">
			///  Sets type of editor. If that option is not set, but the 'value' option is defined, then if type of 'value' is Number, then the 'numeric' type is used, if type of 'value' is Date, then the 'date' type is used. For all other types of 'value' the 'text' type is used. Note: if that option is set to 'datepicker', then application should ensure that css and js files used by jquery.ui.datepicker are available.
			/// </field>
			type: {
				get: function () {
					return $(this._element).igEditor('option', 'type');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'type', value);
				}
			},
			/// <field type="Object">
			/// Sets gets strings used for title of buttons. Value of object should contain pairs or key:value members. Note: any sub-option of locale can appear within the main option of igEditor. In this case those values within main options will have highest priority and override corresponding value in locale.
			/// </field>
			locale: {
				get: function () {
					return $(this._element).igEditor('option', 'locale');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'locale', value);
				}
			},
			/// <field type="Number">
			/// Sets gets width of editor in pixels.
///				Only positive values have effect. If that option is not set, then value of style.width of base html element will be used automatically. However, that value should be in the px units.
			/// </field>
			width: {
				get: function () {
					return $(this._element).igEditor('option', 'width');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'width', value);
				}
			},
			/// <field type="Number">
			/// Sets gets height of editor in pixels.
///				Only positive values have effect.
///				If that option is not set, then value of style.height of base html element will be used automatically. However, that value should be in the px units.
			/// </field>
			height: {
				get: function () {
					return $(this._element).igEditor('option', 'height');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'height', value);
				}
			},
			/// <field type="Object">
			/// Sets gets options supported by igValidator.
///				In order to enable validation and use defaults, an empty object can be used.
///				Note: validation rules of igValidator, such as min, max, minLength, required are not supported, but similar properties of igEditor should be used.
			/// </field>
			validatorOptions: {
				get: function () {
					return $(this._element).igEditor('option', 'validatorOptions');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'validatorOptions', value);
				}
			},
			/// <field type="Bool">
			/// Set gets validation for empty value in editor.
			/// </field>
			required: {
				get: function () {
					return $(this._element).igEditor('option', 'required');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'required', value);
				}
			},
			/// <field type="String">
			/// Sets gets style.display for outer html element.
///				The value of "" will disable changing style.display.
///				If the base html element is not INPUT, TEXTAREA, SPAN or DIV, then it is recommended to set this option to an empty string,
///				because, some browsers may fail to handle display other than default value of browser.
///				For example, if base element for editor is TD, then behavior in Chrome can be problematic.
			/// </field>
			display: {
				get: function () {
					return $(this._element).igEditor('option', 'display');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'display', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets option to wrap input field into SPAN..
///				That option can be set only on initialization and it is available only when base element is INPUT or TEXTAREA.
///				If drop-down button or spin-buttons are enabled or the theme is set, then that option has no effect.
///				The reason for that property is to allow vertical alignment of several igEditor controls when they are located in html inline and some of them have enabled buttons.
///				If all editors are created within SPAN wrapper, then they will appear on the same line. Otherwise, editors in SPANs will be shifted 3-6 pixels above INPUT editors.
			/// </field>
			renderInContainer: {
				get: function () {
					return $(this._element).igEditor('option', 'renderInContainer');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'renderInContainer', value);
				}
			},
			/// <field type="Enumeration">
			///  Sets gets behavior of selection/caret in input-field when editor gets focus.
			/// </field>
			selectionOnFocus: {
				get: function () {
					return $(this._element).igEditor('option', 'selectionOnFocus');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'selectionOnFocus', value);
				}
			},
			/// <field type="Bool">
			/// Set gets ability to modify editor from keyboard (false) or disables keyboard (true).
///				Notes:
///				If the "spinOnReadOnly" is enabled, then value will be modified on spin regardless of the "readOnly".
///				If the "listItems" has items and "dropDownTriggers" is defined, then value will be modified from drop-down list regardless of the "readOnly".
			/// </field>
			readOnly: {
				get: function () {
					return $(this._element).igEditor('option', 'readOnly');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'readOnly', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to override the "readOnly" option and allow changes "value" of editor on spin events. Value false does not allow spin when "readOnly" is true.
			/// </field>
			spinOnReadOnly: {
				get: function () {
					return $(this._element).igEditor('option', 'spinOnReadOnly');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'spinOnReadOnly', value);
				}
			},
			/// <field type="Bool">
			/// If that option is disabled and the editor is not focused, then a mouse click on the spin button will keep the text in the editor in display (not focus) format and perform spin actions.
			/// </field>
			focusOnSpin: {
				get: function () {
					return $(this._element).igEditor('option', 'focusOnSpin');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'focusOnSpin', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to automatically set value in editor to opposite limit, when spin action reached minimum or maximum limit.
///				In case of listOfItems, the first and the last items in list are used as minimum and maximum values.
			/// </field>
			spinWrapAround: {
				get: function () {
					return $(this._element).igEditor('option', 'spinWrapAround');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'spinWrapAround', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to hide the Enter key from browser.
			/// </field>
			hideEnterKey: {
				get: function () {
					return $(this._element).igEditor('option', 'hideEnterKey');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'hideEnterKey', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to override the "readOnly" option and allow showing the drop-down list and changing the value in the editor from a list. Value false does not allow drop-down when "readOnly" is true.
			/// </field>
			dropDownOnReadOnly: {
				get: function () {
					return $(this._element).igEditor('option', 'dropDownOnReadOnly');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'dropDownOnReadOnly', value);
				}
			},
			/// <field type="Enumeration">
			///  Sets gets list of actions which trigger display of drop-down list or calendar.
			/// </field>
			dropDownTriggers: {
				get: function () {
					return $(this._element).igEditor('option', 'dropDownTriggers');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'dropDownTriggers', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets location of drop-down list.
///				Value true will create html element for list as a child of main html element.
///				Value false creates list as a child of body.
///				Notes:
///				That option has effect only for drop-down defined by listOfItems.
///				The value of true is supported only when main (outer) html element of editor is container such as SPAN or DIV, or buttons are enabled, or renderInContainer is enabled, the theme is set.
///				The value of true allows a better positioning and can be useful when editor is located in complex layout and containers with "position:fixed".
			/// </field>
			listDropDownAsChild: {
				get: function () {
					return $(this._element).igEditor('option', 'listDropDownAsChild');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'listDropDownAsChild', value);
				}
			},
			/// <field type="Number">
			/// Sets gets custom width of drop-down list in pixels. If value is equal to 0 or negative, then the width of editor is used.
			/// </field>
			listWidth: {
				get: function () {
					return $(this._element).igEditor('option', 'listWidth');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'listWidth', value);
				}
			},
			/// <field type="Number">
			/// Sets gets maximum height of drop-down list in pixels. If value is equal to 0 or negative, then the height of list is defined by number of items in list.
			/// </field>
			listMaxHeight: {
				get: function () {
					return $(this._element).igEditor('option', 'listMaxHeight');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'listMaxHeight', value);
				}
			},
			/// <field type="Number">
			/// Sets gets number of columns in drop-down list.
			/// </field>
			listColumns: {
				get: function () {
					return $(this._element).igEditor('option', 'listColumns');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'listColumns', value);
				}
			},
			/// <field type="Number">
			/// Sets gets duration of animation in milliseconds when drop-down list or calendar is displayed. Default is 200. If value is less than 5, then animation is disabled.
			/// </field>
			animationShowDuration: {
				get: function () {
					return $(this._element).igEditor('option', 'animationShowDuration');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'animationShowDuration', value);
				}
			},
			/// <field type="Number">
			/// Sets gets duration of animation in milliseconds when drop-down list is hidden. That option is not supported when "type" is set to "datepicker", because the jquery.ui.datepicker does not have similar feature. Default is 200. If value is less than 5, then animation is disabled.
			/// </field>
			animationHideDuration: {
				get: function () {
					return $(this._element).igEditor('option', 'animationHideDuration');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'animationHideDuration', value);
				}
			},
			/// <field type="String">
			/// Gets sets name of (hidden) INPUT element, which value will be set to current value of editor on any change.
			/// </field>
			inputName: {
				get: function () {
					return $(this._element).igEditor('option', 'inputName');
				},
				set: function (value) {
					$(this._element).igEditor('option', 'inputName', value);
				}
			},

			// Public methods
			/*
			destroy: function igEditor_destroy() {
				$(this._element).igEditor('destroy');
			}
			*/
						/* Triggers validation of editor and show error message. That method has effect only when validation is enabled.
			*/
            validate: function Editor_validate (noLabel) {
                $(this._element).igEditor('validate', noLabel);
            },
			/* Checks if value in editor is value. Note: that method may fail if editor has focus (in edit mode).
			*/
            isValid: function Editor_isValid () {
                $(this._element).igEditor('isValid');
            },
			/* Gets calculated value of regional option used by numeric and date editors.
			*/
            getRegionalOption: function Editor_getRegionalOption (name) {
                $(this._element).igEditor('getRegionalOption', name);
            },
			/* Gets reference to jquery object which is used as edit field.
returnType="$" Returns reference to jquery object. That can be INPUT or TEXTAREA.
			*/
            field: function Editor_field () {
                $(this._element).igEditor('field');
            },
			/* Gets reference to jquery object which is used as top/outer element of igEditor.
returnType="$" Returns reference to jquery object.
			*/
            mainElement: function Editor_mainElement () {
                $(this._element).igEditor('mainElement');
            },
			/* Gets reference to jquery object which is used as container of drop-down.
returnType="$" Returns reference to jquery object or null. That can be container of list items or in case of igDatePicker it can be calendar of jquery.datepicker.
			*/
            dropDownElement: function Editor_dropDownElement () {
                $(this._element).igEditor('dropDownElement');
            },
			/* Shows editor (if it was hidden).
			*/
            show: function Editor_show () {
                $(this._element).igEditor('show');
            },
			/* Hides editor.
			*/
            hide: function Editor_hide () {
                $(this._element).igEditor('hide');
            },
			/* Removes editor from its parent element, but keeps the rest of functionality.
			*/
            remove: function Editor_remove () {
                $(this._element).igEditor('remove');
            },
			/* Gets sets visibility of drop-down list.
returnType="bool|object" If parameter is defined, then returns reference to this igCombo, otherwise, returns true if drop down is visible and false if drop down is hidden.
			*/
            dropDownVisible: function Editor_dropDownVisible (showHide) {
                $(this._element).igEditor('dropDownVisible', showHide);
            },
			/* Finds index of list item by text.
			*/
            findListItemIndex: function Editor_findListItemIndex (text, ignoreCase, partial, contains) {
                $(this._element).igEditor('findListItemIndex', text, ignoreCase, partial, contains);
            },
			/* Adds several items to list.
			*/
            addListItems: function Editor_addListItems (items, index) {
                $(this._element).igEditor('addListItems', items, index);
            },
			/* Adds item to list.
			*/
            addListItem: function Editor_addListItem (item, index) {
                $(this._element).igEditor('addListItem', item, index);
            },
			/* Removes item from list.
			*/
            removeListItem: function Editor_removeListItem (item) {
                $(this._element).igEditor('removeListItem', item);
            },
			/* Removes item from list at index.
			*/
            removeListItemAt: function Editor_removeListItemAt (index) {
                $(this._element).igEditor('removeListItemAt', index);
            },
			/* Removes all items from list.
			*/
            clearListItems: function Editor_clearListItems () {
                $(this._element).igEditor('clearListItems');
            },
			/* Gets sets selected index of list item.
			*/
            selectedListIndex: function Editor_selectedListIndex (index) {
                $(this._element).igEditor('selectedListIndex', index);
            },
			/* Gets reference to selected item in list.
returnType="string|number|date|object" Returns reference to selected item or null.
			*/
            getSelectedListItem: function Editor_getSelectedListItem () {
                $(this._element).igEditor('getSelectedListItem');
            },
			/* Checks if invalid message is displayed.
			*/
            hasInvalidMessage: function Editor_hasInvalidMessage () {
                $(this._element).igEditor('hasInvalidMessage');
            },
			/* Gets reference to igValidator used by igEditor.
			*/
            validator: function Editor_validator () {
                $(this._element).igEditor('validator');
            },
			/* Gets sets text in editor.
returnType="string|object" Returns text in editor if parameter is undefined. Otherwise, it returns reference to this igEditor.
			*/
            text: function Editor_text (val, flag) {
                $(this._element).igEditor('text', val, flag);
            },
			/* Gets sets value in editor.
returnType="string|number|date|object" Returns value in editor if parameter is undefined. Otherwise, it returns reference to this igEditor.
			*/
            value: function Editor_value (val) {
                $(this._element).igEditor('value', val);
            },
			/* Paste text at location of caret. Note: method raises the "textChanged" event.
			*/
            paste: function Editor_paste (txt, flag) {
                $(this._element).igEditor('paste', txt, flag);
            },
			/* Selects text in editor. If parameters are equal, then than method sets location of caret. That method has effect only when editor has focus.
			*/
            select: function Editor_select (sel0, sel1, val) {
                $(this._element).igEditor('select', sel0, sel1, val);
            },
			/* Gets selected text in editor.
			*/
            getSelectedText: function Editor_getSelectedText () {
                $(this._element).igEditor('getSelectedText');
            },
			/* Gets left or right edge of selection. That method can be used only when editor has focus.
			*/
            getSelection: function Editor_getSelection (start) {
                $(this._element).igEditor('getSelection', start);
            },
			/* Set focus to editor with delay.
			*/
            setFocus: function Editor_setFocus (delay) {
                $(this._element).igEditor('setFocus', delay);
            },
			/* Checks if editor has focus.
			*/
            hasFocus: function Editor_hasFocus () {
                $(this._element).igEditor('hasFocus');
            },
			/* Get value in editor by dataMode.
returnType="string|number|date|object" Returns value in editor or null.
			*/
            getValueByMode: function Editor_getValueByMode (mode, v, getVal) {
                $(this._element).igEditor('getValueByMode', mode, v, getVal);
            },
			/* Increments of decrements value in editor according to the parameter.
If editor has listItems, then that method increments or decrements selected index in list and sets value in editor to new selected item.
In this case if delta is positive, then list selected index in incremented by 1 and if delta is negative then selected index is decremented by 1.
			*/
            spin: function Editor_spin (delta) {
                $(this._element).igEditor('spin', delta);
            },
			/* Destroys editor and return base element back to its pre-init state.
			*/
            destroy: function Editor_destroy () {
                $(this._element).igEditor('destroy');
            }

        }, {
			// Static members
		})
    });
}());