// igHierarchicalGrid extension
/*global WinJS, $*/
(function () {
    WinJS.Namespace.define('Infragistics.UI', {
        HierarchicalGrid: WinJS.Class.define(function (element, options) {
			// This needs to be defined for all widgets
            if (!element) {
                throw new Error("The Infragistics control requires a DOM element to be instantiated on.");
            }
			this._element = element;
			// Initializes the control
			MSApp.execUnsafeLocalFunction(function () {
				$(element).igHierarchicalGrid(options);
			});
			this.element.winControl = this;
        }, {
			// Private properties
			_element: null,
            // Public options
			// The element option is generic for all widget and is only a getter
			element: {
                get: function () {
                    return this._element;
                }
            },
						/// <field type="Number">
			/// Only the first level will be data-bound initially. Also serves as "render" depth, meaning that depending on this prop, the grid will
			/// </field>
			initialDataBindDepth: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'initialDataBindDepth');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'initialDataBindDepth', value);
				}
			},
			/// <field type="Number">
			/// No levels will be automatically expanded when the widget is instantiated for the first time
			/// </field>
			initialExpandDepth: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'initialExpandDepth');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'initialExpandDepth', value);
				}
			},
			/// <field type="Bool">
			/// If true, encodes all requests using OData conventions and the $expand syntax
			/// </field>
			odata: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'odata');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'odata', value);
				}
			},
			/// <field type="Bool">
			/// If true, load on demand will be achieved using REST compliant resource access with appropriate url-s for
///				the ajax GET calls.
			/// </field>
			rest: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'rest');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'rest', value);
				}
			},
			/// <field type="Number">
			/// Specifies the limit on the number of levels to bind to
			/// </field>
			maxDataBindDepth: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'maxDataBindDepth');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'maxDataBindDepth', value);
				}
			},
			/// <field type="String">
			/// Specifies the default property in the response where children will be located
			/// </field>
			defaultChildrenDataProperty: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'defaultChildrenDataProperty');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'defaultChildrenDataProperty', value);
				}
			},
			/// <field type="Bool">
			/// if true, will autogenerate all layouts assuming default values for "childrenDataProperty" 
///				When autoGenerateLayouts is enabled, it will automatically generate all columns recursively. (all columns on all levels)
			/// </field>
			autoGenerateLayouts: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'autoGenerateLayouts');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'autoGenerateLayouts', value);
				}
			},
			/// <field type="Bool">
			/// applies a linear animation - either expanding or collapsing depending on the parent row state
			/// </field>
			expandCollapseAnimations: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'expandCollapseAnimations');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'expandCollapseAnimations', value);
				}
			},
			/// <field type="Number">
			/// Specifies the expand column width
			/// </field>
			expandColWidth: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'expandColWidth');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'expandColWidth', value);
				}
			},
			/// <field type="String">
			/// Specifies the delimiter for constructing paths , for hierarchical lookup of data
			/// </field>
			pathSeparator: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'pathSeparator');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'pathSeparator', value);
				}
			},
			/// <field type="Number">
			/// The row expanding/collapsing animation duration in ms.
			/// </field>
			animationDuration: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'animationDuration');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'animationDuration', value);
				}
			},
			/// <field type="String">
			/// Specifies the default tooltip applied to an expand column cell, that is currently collapsed
			/// </field>
			expandTooltip: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'expandTooltip');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'expandTooltip', value);
				}
			},
			/// <field type="String">
			/// Specifies the default tooltip applied to an expand column cell, that is currently expanded
			/// </field>
			collapseTooltip: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'collapseTooltip');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'collapseTooltip', value);
				}
			},
			/// <field type="Array">
			/// An array of column objects
			/// </field>
			columns: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'columns');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'columns', value);
				}
			},
			/// <field type="Object">
			/// List of columnLayout objects that specify the structure of the child grids. All options that are applicable to a flat grid are also applicable here
			/// </field>
			columnLayouts: {
				get: function () {
					return $(this._element).igHierarchicalGrid('option', 'columnLayouts');
				},
				set: function (value) {
					$(this._element).igHierarchicalGrid('option', 'columnLayouts', value);
				}
			},

			// Public methods
			/*
			destroy: function igHierarchicalGrid_destroy() {
				$(this._element).igHierarchicalGrid('destroy');
			}
			*/
						/* Data binds the hierarchical grid. No child grids will be created or rendered by default, unless there is initialExpandDepth >= 0 set.
			*/
            dataBind: function HierarchicalGrid_dataBind () {
                $(this._element).igHierarchicalGrid('dataBind');
            },
			/* returns the element of the root grid (igGrid)
			*/
            root: function HierarchicalGrid_root () {
                $(this._element).igHierarchicalGrid('root');
            },
			/* returns the widget object of the root grid (igGrid)
			*/
            rootWidget: function HierarchicalGrid_rootWidget () {
                $(this._element).igHierarchicalGrid('rootWidget');
            },
			/* returns a flat list of all child grid widgets (not elements) - recursive
			*/
            allChildrenWidgets: function HierarchicalGrid_allChildrenWidgets () {
                $(this._element).igHierarchicalGrid('allChildrenWidgets');
            },
			/* returns a flat list of all child grid elements (recursive)
			*/
            allChildren: function HierarchicalGrid_allChildren () {
                $(this._element).igHierarchicalGrid('allChildren');
            },
			/* expands or collapses (toggles) a parent row
			*/
            toggle: function HierarchicalGrid_toggle (element) {
                $(this._element).igHierarchicalGrid('toggle', element);
            },
			/* expands (toggles) a parent row
			*/
            expand: function HierarchicalGrid_expand (id) {
                $(this._element).igHierarchicalGrid('expand', id);
            },
			/* checks if a parent row is expanded or not
			*/
            expanded: function HierarchicalGrid_expanded (element) {
                $(this._element).igHierarchicalGrid('expanded', element);
            },
			/* collapses a parent row
			*/
            collapse: function HierarchicalGrid_collapse (id) {
                $(this._element).igHierarchicalGrid('collapse', id);
            },
			/* checks if a parent row is currently collapsed
			*/
            collapsed: function HierarchicalGrid_collapsed (element) {
                $(this._element).igHierarchicalGrid('collapsed', element);
            },
			/* checks if a parent row is populated with data
			*/
            populated: function HierarchicalGrid_populated (element) {
                $(this._element).igHierarchicalGrid('populated', element);
            },
			/* Commits pending transactions to the client data source for main and all child grids.
			*/
            commit: function HierarchicalGrid_commit () {
                $(this._element).igHierarchicalGrid('commit');
            },
			/* Clears the transaction log (delegates to igDataSource). Note that this does not update the UI. In case the UI must be updated, set the second parameter "updateUI" to true, which will trigger a call to dataBind() to re-render the contents.
			*/
            rollback: function HierarchicalGrid_rollback (rebind) {
                $(this._element).igHierarchicalGrid('rollback', rebind);
            },
			/* posts to the settings.updateUrl using $.ajax, by serializing the changes as url params
			*/
            saveChanges: function HierarchicalGrid_saveChanges () {
                $(this._element).igHierarchicalGrid('saveChanges');
            },
			/* destroys the hierarchical grid by recursively destroying all child grids
			*/
            destroy: function HierarchicalGrid_destroy () {
                $(this._element).igHierarchicalGrid('destroy');
            }

        }, {
			// Static members
		})
    });
}());