// igMaskEditor extension
/*global WinJS, $*/
(function () {
    WinJS.Namespace.define('Infragistics.UI', {
        MaskEditor: WinJS.Class.define(function (element, options) {
			// This needs to be defined for all widgets
            if (!element) {
                throw new Error("The Infragistics control requires a DOM element to be instantiated on.");
            }
			this._element = element;
			// Initializes the control
			MSApp.execUnsafeLocalFunction(function () {
				$(element).igMaskEditor(options);
			});
			this.element.winControl = this;
        }, {
			// Private properties
			_element: null,
            // Public options
			// The element option is generic for all widget and is only a getter
			element: {
                get: function () {
                    return this._element;
                }
            },
						/// <field type="String">
			/// Sets gets ability to enter only specific characters in input-field from keyboard and on paste.
///				Notes:
///				If "excludeKeys" option contains same characters as this option, then "excludeKeys" has priority.
///				Letters should be set in upper case.
///				Making difference between upper and lower case is not supported.
			/// </field>
			includeKeys: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'includeKeys');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'includeKeys', value);
				}
			},
			/// <field type="String">
			/// Sets gets ability to prevent entering specific characters from keyboard or on paste.
///				Notes:
///				If a character is specified in "includeKeys" option also, then "excludeKeys" has priority.
///				Letters should be set in upper case.
///				Making difference between upper and lower case is not supported.
			/// </field>
			excludeKeys: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'excludeKeys');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'excludeKeys', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to convert input characters to upper case (true) or keeps characters as they are (false). That option has effect only while keyboard entries and paste.
			/// </field>
			toUpper: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'toUpper');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'toUpper', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to convert input characters to lower case (true) or keeps characters as they are (false). That option has effect only while keyboard entries and paste.
			/// </field>
			toLower: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'toLower');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'toLower', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets case validation. Value of false enables and true disables validation for case of entries to match with items in listOfItems.
///				That option has effect only when "listMatchOnly" or "listAutoComplete" options are enabled.
			/// </field>
			listMatchIgnoreCase: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'listMatchIgnoreCase');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'listMatchIgnoreCase', value);
				}
			},
			/// <field type="Bool">
			/// Set gets list match.
///				Value true enables and false disables validation of keyboard entries to match with items in listOfItems.
///				If that option is enabled and entered text does not match with a part of any item in list, then entry is canceled.
///				Notes:
///				That option is supported only when the "type" of editor is "text" and it has effect only when listOfItems is set.
///				Partial match can be validated from the beginning of an item or anywhere within an item. That is defined by the "listMatchContains" option.
///				Validation can be case sensitive or not and that is defined by the "listMatchIgnoreCase" option.
			/// </field>
			listMatchOnly: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'listMatchOnly');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'listMatchOnly', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets list match contains.
///				Value true enables and false disables validation for partial match of entered text anywhere within an item in the listOfItems or only from the beginning of an item.
///				That option has effect only when "listMatchOnly" or "listAutoComplete" options are enabled.
			/// </field>
			listMatchContains: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'listMatchContains');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'listMatchContains', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets autocomplete.
///				Value true enables and false disables auto-complete functionality to fill value of editor by a partially matching item from the listOfItems.
///				If that option is enabled and entered text does not match with a part of any item in list, then original value is preserved.
///				Notes:
///				That option is supported only when the "type" of editor is "text" and it has effect only when listOfItems is set.
///				Partial match can be validated from the beginning of an item or anywhere within an item. That is defined by the "listMatchContains" option.
///				Validation can be case sensitive or not and that is defined by the "listMatchIgnoreCase" option.
			/// </field>
			listAutoComplete: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'listAutoComplete');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'listAutoComplete', value);
				}
			},
			/// <field type="String">
			/// Sets gets input mask. Mask may include filter-flags and literal characters.
///				Literal characters are part of mask which cannot be modified by end user. In order to use a filter-flag as a literal character, the escape "\\" character should be used.
///				Default is "CCCCCCCCCC"
///				Note: optional flags/entries affect the value returned by get of the "value" and "text" methods.
///				List of filter-flags:
///				C: any keyboard character. Entry is optional.
///				&: any keyboard character. Entry is required.
///				a: letter or digit character. Entry is optional.
///				A: letter or digit character. Entry is required.
///				?: letter character. Entry is optional.
///				L: letter character. Entry is required.
///				9: digit character. Entry is optional.
///				0: digit character. Entry is required.
///				#: digit character or "+" or "_". Entry is optional with replacement by "emptyPositionChar" or by "padChar".
///				>: all letters to the right are converted to the upper case. In order to disable conversion, the ">" flag should be used again.
///				<: all letters to the right are converted to the lower case. In order to disable conversion, the "<" flag should be used again.
			/// </field>
			inputMask: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'inputMask');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'inputMask', value);
				}
			},
			/// <field type="String">
			/// Sets gets character which is used as prompt in edit mode for available entry position.
			/// </field>
			promptChar: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'promptChar');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'promptChar', value);
				}
			},
			/// <field type="String">
			/// Sets gets character which is used as replacement of not-filled required position in mask when editor is in display mode (not focused).
			/// </field>
			padChar: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'padChar');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'padChar', value);
				}
			},
			/// <field type="String">
			/// Sets gets character which is used as replacement of not-filled required position in mask when application calls get for the "value" or for the "text" methods.
			/// </field>
			emptyChar: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'emptyChar');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'emptyChar', value);
				}
			},
			/// <field type="Enumeration">
			///  Sets gets type of value returned by the get of value() method. That also affects functionality of the set value(val) method and the copy/paste operations of browser.
			/// </field>
			dataMode: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'dataMode');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'dataMode', value);
				}
			},
			/// <field type="Bool">
			/// Set gets ability to hide mask in focus state.
///				Value of false allows to show all input mask with prompt characters regardless if available mask positions are filled or not.
///				Value of true will show only part of mask located on the left side from caret when available mask positions on the right side from caret are empty.
			/// </field>
			hideMaskOnFocus: {
				get: function () {
					return $(this._element).igMaskEditor('option', 'hideMaskOnFocus');
				},
				set: function (value) {
					$(this._element).igMaskEditor('option', 'hideMaskOnFocus', value);
				}
			},

			// Public methods
			/*
			destroy: function igMaskEditor_destroy() {
				$(this._element).igMaskEditor('destroy');
			}
			*/
			
        }, {
			// Static members
		})
    });
}());