// igNumericEditor extension
/*global WinJS, $*/
(function () {
    WinJS.Namespace.define('Infragistics.UI', {
        NumericEditor: WinJS.Class.define(function (element, options) {
			// This needs to be defined for all widgets
            if (!element) {
                throw new Error("The Infragistics control requires a DOM element to be instantiated on.");
            }
			this._element = element;
			// Initializes the control
			MSApp.execUnsafeLocalFunction(function () {
				$(element).igNumericEditor(options);
			});
			this.element.winControl = this;
        }, {
			// Private properties
			_element: null,
            // Public options
			// The element option is generic for all widget and is only a getter
			element: {
                get: function () {
                    return this._element;
                }
            },
						/// <field type="Object">
			/// Sets gets custom regional settings for editor. If it is string, then $.ig.regional[stringValue] is assumed.
			/// </field>
			regional: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'regional');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'regional', value);
				}
			},
			/// <field type="String">
			/// Sets gets the pattern for negative numeric values, which is used in display (no focus) state.
///				The "n" flag represents the value of number. The "-" and "()" flags are static part of pattern.
///				Note: this option has priority over possible regional settings.
			/// </field>
			negativePattern: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'negativePattern');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'negativePattern', value);
				}
			},
			/// <field type="String">
			/// Sets gets the character, which is used for negative numeric values.
///				Note: this option has priority over possible regional settings.
			/// </field>
			negativeSign: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'negativeSign');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'negativeSign', value);
				}
			},
			/// <field type="String">
			/// Sets gets the character, which is used as decimal separator.
///				Note: this option has priority over possible regional settings.
			/// </field>
			decimalSeparator: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'decimalSeparator');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'decimalSeparator', value);
				}
			},
			/// <field type="String">
			/// Sets gets the character, which is used as separator for groups (like thousands).
///				That option has effect only in display (no focus) state.
///				Note: this option has priority over possible regional settings.
			/// </field>
			groupSeparator: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'groupSeparator');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'groupSeparator', value);
				}
			},
			/// <field type="Object">
			/// (array of number objects) Sets gets the number of digits in integer part of number, which are divided into groups.
///				The "numericGroupSeparator" is inserted between groups.
///				If the sum of all values in array is smaller than the length of integer part, then the last item in array is used for all following groups.
///				Count of groups starts from the decimal point (from right to left).
///				That option has effect only in display (no focus) state.
///				Note: this option has priority over possible regional settings.
			/// </field>
			groups: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'groups');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'groups', value);
				}
			},
			/// <field type="Number">
			/// Sets gets the maximum number of decimal places which are used in display (no focus) state.
///				Note: this option has priority over possible regional settings.
			/// </field>
			maxDecimals: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'maxDecimals');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'maxDecimals', value);
				}
			},
			/// <field type="Number">
			/// Sets gets the minimum number of decimal places which are used in display (no focus) state.
///				If number of digits in fractional part of number is less than the value of this option, then the "0" characters are used to fill missing digits.
///				Note: this option has priority over possible regional settings.
			/// </field>
			minDecimals: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'minDecimals');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'minDecimals', value);
				}
			},
			/// <field type="Number">
			/// Sets gets the minimum value which can be entered in editor by end user.
			/// </field>
			minValue: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'minValue');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'minValue', value);
				}
			},
			/// <field type="Number">
			/// Sets gets the maximum value which can be entered in editor by end user.
			/// </field>
			maxValue: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'maxValue');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'maxValue', value);
				}
			},
			/// <field type="Enumeration">
			/// 
			/// </field>
			scientificFormat: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'scientificFormat');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'scientificFormat', value);
				}
			},
			/// <field type="Number">
			/// Sets gets the representation of null value.
			/// </field>
			nullValue: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'nullValue');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'nullValue', value);
				}
			},
			/// <field type="Number">
			/// Sets gets delta-value which is used to increment or decrement value in editor on spin events.
			/// </field>
			spinDelta: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'spinDelta');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'spinDelta', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to prevent null value.
///				If that option is disabled, and editor has no value, then value is set to 0 (or minValue/maxValue).
			/// </field>
			nullable: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'nullable');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'nullable', value);
				}
			},
			/// <field type="Number">
			/// Sets gets maximum length of text which can be entered by user.
///				Negative values or 0 disables that behavior.
			/// </field>
			maxLength: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'maxLength');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'maxLength', value);
				}
			},
			/// <field type="Enumeration">
			///  Sets gets type of value returned by the get of value() method. That also affects functionality of the set value(val) method and the copy/paste operations of browser.
			/// </field>
			dataMode: {
				get: function () {
					return $(this._element).igNumericEditor('option', 'dataMode');
				},
				set: function (value) {
					$(this._element).igNumericEditor('option', 'dataMode', value);
				}
			},

			// Public methods
			/*
			destroy: function igNumericEditor_destroy() {
				$(this._element).igNumericEditor('destroy');
			}
			*/
			
        }, {
			// Static members
		})
    });
}());