// igTextEditor extension
/*global WinJS, $*/
(function () {
    WinJS.Namespace.define('Infragistics.UI', {
        TextEditor: WinJS.Class.define(function (element, options) {
			// This needs to be defined for all widgets
            if (!element) {
                throw new Error("The Infragistics control requires a DOM element to be instantiated on.");
            }
			this._element = element;
			// Initializes the control
			MSApp.execUnsafeLocalFunction(function () {
				$(element).igTextEditor(options);
			});
			this.element.winControl = this;
        }, {
			// Private properties
			_element: null,
            // Public options
			// The element option is generic for all widget and is only a getter
			element: {
                get: function () {
                    return this._element;
                }
            },
						/// <field type="Enumeration">
			///  Sets gets text mode of editor such as: single-line text editor, password editor or multiline editor.
			/// </field>
			textMode: {
				get: function () {
					return $(this._element).igTextEditor('option', 'textMode');
				},
				set: function (value) {
					$(this._element).igTextEditor('option', 'textMode', value);
				}
			},
			/// <field type="Number">
			/// Sets gets maximum length of text which can be entered by user.
///				Negative values or 0 disables that behavior.
			/// </field>
			maxLength: {
				get: function () {
					return $(this._element).igTextEditor('option', 'maxLength');
				},
				set: function (value) {
					$(this._element).igTextEditor('option', 'maxLength', value);
				}
			},
			/// <field type="String">
			/// Sets gets ability to enter only specific characters in input-field from keyboard and on paste.
///				Notes:
///				If "excludeKeys" option contains same characters as this option, then "excludeKeys" has priority.
///				Letters should be set in upper case.
///				Making difference between upper and lower case is not supported.
			/// </field>
			includeKeys: {
				get: function () {
					return $(this._element).igTextEditor('option', 'includeKeys');
				},
				set: function (value) {
					$(this._element).igTextEditor('option', 'includeKeys', value);
				}
			},
			/// <field type="String">
			/// Sets gets ability to prevent entering specific characters from keyboard or on paste.
///				Notes:
///				If a character is specified in "includeKeys" option also, then "excludeKeys" has priority.
///				Letters should be set in upper case.
///				Making difference between upper and lower case is not supported.
			/// </field>
			excludeKeys: {
				get: function () {
					return $(this._element).igTextEditor('option', 'excludeKeys');
				},
				set: function (value) {
					$(this._element).igTextEditor('option', 'excludeKeys', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to convert input characters to upper case (true) or keeps characters as they are (false). That option has effect only while keyboard entries and paste.
			/// </field>
			toUpper: {
				get: function () {
					return $(this._element).igTextEditor('option', 'toUpper');
				},
				set: function (value) {
					$(this._element).igTextEditor('option', 'toUpper', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets ability to convert input characters to lower case (true) or keeps characters as they are (false). That option has effect only while keyboard entries and paste.
			/// </field>
			toLower: {
				get: function () {
					return $(this._element).igTextEditor('option', 'toLower');
				},
				set: function (value) {
					$(this._element).igTextEditor('option', 'toLower', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets case validation. Value of false enables and true disables validation for case of entries to match with items in listOfItems.
///				That option has effect only when "listMatchOnly" or "listAutoComplete" options are enabled.
			/// </field>
			listMatchIgnoreCase: {
				get: function () {
					return $(this._element).igTextEditor('option', 'listMatchIgnoreCase');
				},
				set: function (value) {
					$(this._element).igTextEditor('option', 'listMatchIgnoreCase', value);
				}
			},
			/// <field type="Bool">
			/// Set gets list match.
///				Value true enables and false disables validation of keyboard entries to match with items in listOfItems.
///				If that option is enabled and entered text does not match with a part of any item in list, then entry is canceled.
///				Notes:
///				That option is supported only when the "type" of editor is "text" and it has effect only when listOfItems is set.
///				Partial match can be validated from the beginning of an item or anywhere within an item. That is defined by the "listMatchContains" option.
///				Validation can be case sensitive or not and that is defined by the "listMatchIgnoreCase" option.
			/// </field>
			listMatchOnly: {
				get: function () {
					return $(this._element).igTextEditor('option', 'listMatchOnly');
				},
				set: function (value) {
					$(this._element).igTextEditor('option', 'listMatchOnly', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets list match contains.
///				Value true enables and false disables validation for partial match of entered text anywhere within an item in the listOfItems or only from the beginning of an item.
///				That option has effect only when "listMatchOnly" or "listAutoComplete" options are enabled.
			/// </field>
			listMatchContains: {
				get: function () {
					return $(this._element).igTextEditor('option', 'listMatchContains');
				},
				set: function (value) {
					$(this._element).igTextEditor('option', 'listMatchContains', value);
				}
			},
			/// <field type="Bool">
			/// Sets gets autocomplete.
///				Value true enables and false disables auto-complete functionality to fill value of editor by a partially matching item from the listOfItems.
///				If that option is enabled and entered text does not match with a part of any item in list, then original value is preserved.
///				Notes:
///				That option is supported only when the "type" of editor is "text" and it has effect only when listOfItems is set.
///				Partial match can be validated from the beginning of an item or anywhere within an item. That is defined by the "listMatchContains" option.
///				Validation can be case sensitive or not and that is defined by the "listMatchIgnoreCase" option.
			/// </field>
			listAutoComplete: {
				get: function () {
					return $(this._element).igTextEditor('option', 'listAutoComplete');
				},
				set: function (value) {
					$(this._element).igTextEditor('option', 'listAutoComplete', value);
				}
			},

			// Public methods
			/*
			destroy: function igTextEditor_destroy() {
				$(this._element).igTextEditor('destroy');
			}
			*/
			
        }, {
			// Static members
		})
    });
}());